'use strict';

const chalk = require('chalk');
const emoji = require('node-emoji');
const inquirer = require('inquirer');
const spawn = require('cross-spawn');
const npm = require('fie-npm');
const semver = require('semver');
const cache = require('fie-cache');
const utils = require('fie-utils');
const fieUser = require('fie-user');
const fieEnv = require('fie-env');
const log = require('fie-log')('core-upgrade');
const Intl = require('fie-intl');
const message = require('../locale/index');

/**
 * 升级提示，若发布新版本，会定时提醒是否需要更新
 * @param data object { "name" : 包名, "version" : 当前版本}
 */
function* updateTip(data) {
  const latest = yield npm.latest(data.name);
  // latest 没有值，可能没有网络
  if (!latest) {
    return;
  }

  // 仅在内网环境下，会对fie本身做灰度处理
  if (fieEnv.isIntranet() && data.name === 'fie') {
    // 先判断是否在名单里，再判断版本
    try {
      const user = fieUser.getUser();
      if (!user || !user.empId) return;
      const fie4GrayResult = yield utils.getFieCoreGrayResult();
      if (fie4GrayResult) {
        const { version, members } = fie4GrayResult;
        if (members && members.indexOf(user.empId) >= 0) {
          if (version && semver.gt(version, data.version)) {
            const installer = getInstaller();
            const installVersion = `fie@${fie4GrayResult.version}`;
            log.info('经检测FIE已发布最新版本，正为你做自动更新...');
            spawn.sync(installer, ['install', '-g', `${installVersion}`], { stdio: 'inherit' });
            return;
          }
        }
      }
    } catch (error) {}
  }
  log.debug('%s current-version: %s, latest-version: %s', data.name, data.version, latest.version);
  if (!semver.lt(data.version, latest.version)) {
    return;
  }
  const localArr = data.version.split('.');
  const latestArr = latest.version.split('.');
  const localX = localArr[0];
  const localY = localArr[1];
  const latestX = latestArr[0];
  const latestY = latestArr[1];
  
  const pkgScope = data.name.indexOf('@ali') !== -1;
  let installer;
  if (pkgScope) {
    installer = 'tnpm';
  } else {
    installer = getInstaller();
  }
  const intl = new Intl(message);
  console.log('\n');
  log.warn(
    `******************** ${emoji.get('warning')} ${emoji.get('warning')}   ${intl.get(
      'updateTips'
    )}  ${emoji.get('warning')} ${emoji.get('warning')} **********************`
  );
  const tips = `${data.name} 最新版本是 ${latest.version}, 本地版本是 ${data.version}`;
  if (localX !== latestX  || localY !== latestY) {
    const answer = yield inquirer.prompt({
      type: 'confirm',
      name: 'autoUpdate',
      message: `${tips},是否为你自动升级到最新版本？`
    });
    if (answer.autoUpdate) {
      spawn.sync(installer, ['install', '-g', `${data.name}`], { stdio: 'inherit' });
    }
  } else {
    log.info(`${tips}, 自动为你升级到最新版本...`);
    spawn.sync(installer, ['install', '-g', `${data.name}`], { stdio: 'inherit' });
  }
}

// 获取安装工具是tnpm, cnpm还是npm
function getInstaller() {
  // 检测是否有tnpm
  const tnpmRes = spawn.sync('tnpm', ['-v']);
  if (!tnpmRes.error) {
    const str = tnpmRes.stdout.toString();
    const matchRes = str.match(/tnpm@/);
    if (matchRes && matchRes.length) {
      return 'tnpm';
    }
  }
  // 检测是否有npm
  const cnpmRes = spawn.sync('cnpm', ['-v']);
  if (cnpmRes.error) {
    const str = cnpmRes.stdout.toString();
    const matchRes = str.match(/cnpm@/);
    if (matchRes && matchRes.length) {
      return 'cnpm';
    }
  }
  // 都没有，则用npm
  return 'npm';
}

module.exports = updateTip;
